import React, { Component } from 'react';
import { connect } from 'react-redux';
require('./array');
import {
    Container,
    Header,
    List,
    ListItem,
    InputGroup,
    Input,
    Title,
    Content,
    Footer,
    FooterTab,
    Button,
    Picker,
    Icon,
    Text
} from 'native-base';
import * as Titles from './titles';
import * as Common from './common';
import * as UIA from './actions/uiActions';
import { Colors } from './colors';

export const FirstName = 'firstName';
export const LastName = 'lastName';
export const BusinessName = 'businessName';
export const SupportEmail = 'supportEmail';
export const StripeAccountLegalEntityOptions = () => {
    return {
        //'email': validation(['email_empty']),
        [FirstName]: validation(['notempty', 'alphanumericlike', 'maxlength22']),
        [LastName]: validation(['notempty', 'alphanumericlike', 'maxlength22']),
        'maidenName': validation(['notempty', 'alphanumericlike', 'maxlength22']),
        'personalIdNumber': validation(['notempty', 'length9']),
        'accountNumber': validation(['notempty', 'maxlength22']),
        'url': validation(['url_empty']),
        [BusinessName]: validation(['notempty', 'maxlength50']),
        'businessUrl': validation(['url_empty']),
        [AddressCity]: validation(['notempty', 'maxlength50', 'alphanumericlike']),
        [AddressLine1]: validation(['notempty', 'maxlength50', 'alphanumericlike']),
        'productDescription': validation(['maxlength50', 'alphanumericlike']),
        'statementDescriptor': validation(['maxlength22', 'alphanumericlike']),
        [SupportEmail]: validation(['email_empty']),
        'supportUrl': validation(['url_empty']),
        [AddressPostalCode]: validation(['numbers', 'length5']),
        'endDate': validation(['pastdate']),
    };
}
export const AddressPostalCode = 'addressPostalCode';
export const AddressCity = 'addressCity';
export const AddressZip = 'addressZip';
export const AddressLine1 = 'addressLine1';
export function ValidatedCreditCard() {
    return {
        'name': validation(['notempty', 'alphanumericlike', 'maxlength50']),
        'email': validation(['email_empty']),
        'number': validation(['notempty', 'length14t19', 'credit']),
        'cvc': validation(['notempty', 'length3t4']),
        [AddressLine1]: validation(['maxlength50']),
        'addressLine2': validation(['maxlength50']),
        [AddressCity]: validation(['alphanumericlike']),
        [AddressZip]: validation(['zipempty']),
        'expirationYear': validation(['expirationYear'], {
            year: 'expirationYear',
            month: 'expirationMonth'
        }),
        'expirationMonth': validation(['expirationMonth'], {
            year: 'expirationYear',
            month: 'expirationMonth'
        })
    }
}
export function ValidateCreditCard(me, state) {
    var _validation = ValidatedCreditCard();
    var valid = validate(me, _validation, state);

    me.props.UI(UIA.CREDIT_CARD_VALID, valid);

    return valid;
}

export const Email = 'email';
export function ValidateStripeManagedAccount(me, stripeState) {
    var _validation = {
        [Email]: validation(['email_empty']),
        [BusinessName]: validation(['notempty', 'maxlength50', 'alphanumericlike']),
        'businessUrl': validation(['url_empty']),
        'productDescription': validation(['notempty', 'maxlength50']),
        'statementDescriptor': validation(['notempty', 'maxlength22']),
        'supportEmail': validation(['email_empty']),
        'supportUrl': validation(['url_empty']),
    }

    var valid = validate(me, _validation, stripeState);
    me.props.UI(LA.ADD_BANK_ACCOUNT_STATE_VALID, valid, true);
    return valid;
}

export const PersonalAddressCity = 'personalAddressCity';
export const PersonalAddressPostalCode = 'personalAddressPostalCode';
export const PersonalAddressLine1 = 'personalAddressLine1';
export function ValidateStripeAccountPersonalAddress(me, stripeState) {

    var _validation = {
        [PersonalAddressLine1]: validation(['notempty', 'maxlength50', 'alphanumericlike']),
        [PersonalAddressCity]: validation(['notempty', 'maxlength50', 'alphanumericlike']),
        [PersonalAddressPostalCode]: validation(['zip', 'notempty', 'numbers']),
    }

    var valid = validate(me, _validation, stripeState);
    me.props.UI(LA.ADD_STRIPE_ACCOUNT_LEGAL_ENTITY_OPTIONS_STATE_VALID, valid);

    me.props.UI(LA.ADD_STRIPE_ACCOUNT_LEGAL_ENTITY_OPTIONS_STATE, Object.assign({}, stripeState), true);

}
export function ValidateStripeAccountLegalEntityOptions(me, stripeState) {
    var _validation = StripeAccountLegalEntityOptions();
    var valid = validate(me, _validation, stripeState);

    me.props.UI(LA.ADD_STRIPE_ACCOUNT_LEGAL_ENTITY_OPTIONS_STATE_VALID, valid);
    me.props.UI(LA.ADD_STRIPE_ACCOUNT_LEGAL_ENTITY_OPTIONS_STATE, Object.assign({}, stripeState), true);

    return valid;
}

export function validateBank(me) {

    var { state } = me.props;

    var ADD_STRIPE_ACCOUNT_BANK_ACCOUNT_OPTIONS = UIA.Get(state, LA.ADD_STRIPE_ACCOUNT_BANK_ACCOUNT_OPTIONS) || {};
    var ADD_BANK_ACCOUNT_STATE = UIA.Get(state, LA.ADD_BANK_ACCOUNT_STATE);
    var ADD_STRIPE_ACCOUNT_LEGAL_ENTITY_OPTIONS_STATE = UIA.Get(state, LA.ADD_STRIPE_ACCOUNT_LEGAL_ENTITY_OPTIONS_STATE) || {};

    return ValidateStripeAccountBankAccountOptions(me, ADD_STRIPE_ACCOUNT_BANK_ACCOUNT_OPTIONS) &&
        ValidateStripeManagedAccount(me, ADD_BANK_ACCOUNT_STATE) &&
        ValidateStripeAccountLegalEntityOptions(me, ADD_STRIPE_ACCOUNT_LEGAL_ENTITY_OPTIONS_STATE);
}

export function ValidateStripeAccountBankAccountOptions(me, stripeState) {
    var _validation = {
        'accountNumber': validation(['notempty', 'numbers', 'length9to17']),
        'bankName': validation(['notempty', 'alphanumericlike']),
        'accountHolderName': validation(['notempty', 'alphanumericlike']),
        'routingNumber': validation(['notempty', 'numbers', 'length9']),
    }

    var valid = validate(me, _validation, stripeState);
    me.props.UI(LA.ADD_STRIPE_ACCOUNT_BANK_ACCOUNT_OPTIONS_VALID, valid);
    me.props.UI(LA.ADD_STRIPE_ACCOUNT_BANK_ACCOUNT_OPTIONS, stripeState, true);
    return valid;
}

function numbers(x) {
    return !(x || '').toString().split('').find(t => {
        return '0123456789'.indexOf(t) === -1;
    });
}

export function validateUI(ui, obj) {
    if (ui && ui.properties) {
        var _validation = {};
        for (var prop in ui.properties) {
            var vrules = [];
            (ui.properties[prop] || []).map(t => {
                vrules = [...vrules, ...(t.validationRules || [])];
            })
            if (vrules && vrules.length)
                _validation[prop] = validation(vrules);
        }

        var stripeState = obj;
        var valid = validate(_validation, stripeState, (t, v) => {
            return t[v];
        });
        return valid;
    }
}
export function validation(args, opts) {

    return args.map(x => {
        var temp = (function () {
            switch (x) {
                case 'zipempty':
                    return {
                        func: x => !x || (x.length === 5 && numbers(x)),
                        message: Titles.ZipCodeInvalid
                    }

                case 'zip':
                    return {
                        func: x => x.length === 5 && numbers(x),
                        message: Titles.ZipCodeInvalid
                    }
                case 'pastdate':
                case 'beforenow':
                    return {
                        func: v => {
                            try {
                                var date = new Date(Date.parse(v));
                                return Date.now() - date.getTime() > 0;
                            }
                            catch (e) { return false }
                        },
                        message: Titles.MustBeInThePast
                    }
                case '18plus':
                    return {
                        func: v=>{
                            try {
                                var date = new Date(Date.parse(v));
                                return Date.now() - date.getTime() > 5.676e+11;
                            }
                            catch (e) { return false }
                            
                        },
                        message: Titles.MustBeAtLeast18
                    }
                case 'expirationYear':
                    return {
                        func: (v, obj) => {
                            if (obj && opts && opts.month && opts.year) {
                                var date = new Date();
                                var month = date.getUTCMonth() + 1;
                                var year = date.getUTCFullYear();

                                if (year > parseFloat(obj[opts.year])) {
                                    return false;
                                }
                            }
                            return true;
                        },
                        message: Titles.InvalidYear
                    }
                case 'expirationMonth':
                    return {
                        func: (v, obj) => {
                            if (obj && opts && opts.month && opts.year) {
                                var date = new Date();
                                var month = date.getUTCMonth() + 1;
                                var year = date.getUTCFullYear();
                                if ((month > parseFloat(obj[opts.month])) && year === parseFloat(obj[opts.year])) {
                                    return false;
                                }
                                if (year > parseFloat(obj[opts.year])) {
                                    return false;
                                }
                            }
                            return true;
                        },
                        message: Titles.InvalidMonth
                    }
                case 'email':
                    return {
                        func: v => validateEmail(v),
                        message: Titles.EnterValidEmail,
                    }
                case 'maxlength22':
                    return {
                        func: x => x.length <= 22,
                        message: Titles.TooLong,
                    }
                case 'length16':
                    return {
                        func: x => x.length === 16,
                        message: Titles.IncorrectLength,
                    }
                case 'length14t19':
                    return {
                        func: x => x.length >= 14 && x.length <= 19,
                        message: Titles.IncorrectLength,
                    }
                case 'credit':
                    return {
                        func: x => {
                            return CredCardValidations.find(j => {
                                return j.startsWith.find(t => x.startsWith(t)) && j.lengths.find(t => x.length == t);
                            })
                        },
                        message: Titles.InvalidCard
                    }
                case 'length3':
                    return {
                        func: x => x.length === 3,
                        message: Titles.IncorrectLength,
                    }

                case 'maxlength50':
                    return {
                        func: x => x.length <= 50,
                        message: Titles.TooLong,
                    }
                case 'email_empty':
                    return {
                        func: v => !v || validateEmail(v),
                        message: Titles.EnterValidEmail,
                    }
                case 'url':
                    return {
                        func: v => isUrlValid(v),
                        message: Titles.ValidUrl
                    }
                case 'url_empty':
                    return {
                        func: v => !(v || '').toString().trim() || isUrlValid(v),
                        message: Titles.ValidUrl,
                    }
                case 'notempty':
                    return {
                        func: v => {
                            if (Array.isArray(v)) {
                                return v.length;
                            }
                            return v
                        },
                        message: Titles.CantBeEmpty
                    };
                case 'length3t4':
                    return {
                        func: x => x.length >= 3 && x.length <= 4,
                        message: Titles.Length3t4
                    }

                case 'length12':
                    return {
                        func: x => x.length === 12,
                        message: Titles.Length12
                    }
                case 'socialsecurity':
                    return {
                        func: x => x && x.length === 9,
                        message: Titles.InvalidSS
                    }
                case 'length9to17':
                    return {
                        func: x =>
                            x.length === 12 ||
                            x.length === 13 ||
                            x.length === 14 ||
                            x.length === 15 ||
                            x.length === 16 ||
                            x.length === 17 ||
                            x.length === 11 ||
                            x.length === 10 ||
                            x.length === 9,
                        message: Titles.Length9to17
                    }
                case 'length8':
                    return {
                        func: x => x.length === 8,
                        message: Titles.Length8
                    }
                case 'length9':
                    return {
                        func: x => x.length === 9,
                        message: Titles.Length9
                    };
                case 'length4':
                    return {
                        func: x => x.length === 4,
                        message: Titles.Length4
                    }
                case 'length5':
                    return {
                        func: x => x.length === 5,
                        message: Titles.Length5
                    }
                case 'numeric':
                case 'numbers':
                    return {
                        func: numbers,
                        message: Titles.OnlyNumbersAllowed
                    };
                case 'alphanumericlike':
                    return {
                        func: x => !(x || '').toString().split('').find(t => {
                            t = t.toLowerCase();
                            return 'abcdefghijk lmnopqrstuvwxyz,.;\'0123456789'.indexOf(t) === -1;
                        }),
                        message: Titles.ValidCharacters
                    };
            }
        })();
        if (temp) {
            Object.assign(temp, { name: x });
        }
        return temp;
    });
}
export function GetCardType(x) {
    return CredCardValidations.find(j => {
        return j.startsWith.find(t => x.startsWith(t)) && j.lengths.find(t => x.length == t);
    });
}

const CredCardValidations = [
    { name: 'American Express', startsWith: ['34', '37'], lengths: [15] },
    { name: 'Diners Club - Carte Blanche', startsWith: ['300', '301', '302', '303', '304', '305'], lengths: [14] },
    { name: 'Diners Club - International', startsWith: ['36'], lengths: [14] },
    { name: 'Diners Club - USA & Canada', startsWith: ['54'], lengths: [16] },
    { name: 'Discover', startsWith: [...[].interpolate(622126, 622925 + 1, x => x).map(x => x.toString()), ...['6011', '644', '645', '646', '647', '648', '649', '65']], lengths: [16, 17, 18, 19] },
    { name: 'InstaPayment', startsWith: [...[637, 638, 639].map(x => x.toString())], lengths: [16] },
    { name: 'JCB', startsWith: [...[].interpolate(3528, 3589 + 1, x => x).map(x => x.toString())], lengths: [16, 17, 18, 19] },
    { name: 'Maestro', startsWith: [...[5018, 5020, 5038, 5893, 6304, 6759, 6761, 6762, 6763].map(x => x.toString())], lengths: [16, 17, 18, 19] },
    { name: 'MasterCard', startsWith: [...[].interpolate(222100, 272099 + 1, x => x.toString()), ...[51, 52, 53, 54, 55].map(x => x.toString())], lengths: [16, 17, 18, 19] },
    { name: 'Visa', startsWith: [...[4].map(x => x.toString())], lengths: [16, 13, 19] },
    { name: 'Visa Electron', startsWith: [...[4026, 417500, 4508, 4844, 4913, 4917].map(x => x.toString())], lengths: [16] }];

export function InputField(value, iconName, dark) {
    return {
        errorIcon: () => {
            if (value) {
                return <Icon name='md-flag' style={{ color: Colors.Color5 }} />
            }
            if (iconName)
                return <Icon name={iconName} style={{ color: Colors.Color5 }} />;
            return null;
        },
        errorText: () => {
            var text = value;
            if (text) {
                return Common.ErrorText(text);
            }
            return null;
        },
    };
}
function validateEmail(email) {
    var re = /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    return re.test(email);
}

function isUrlValid(userInput) {
    var res = userInput.match(/(http(s)?:\/\/.)?(www\.)?[-a-zA-Z0-9@:%._\+~#=]{2,256}\.[a-z]{2,6}\b([-a-zA-Z0-9@:%_\+.~#?&//=]*)/g);
    if (res == null)
        return false;
    else
        return true;
}
export function validate(validation, stripeState, getValue) {
    var result = { result: true };
    var validationState = {};
    for (var key in validation) {
        var rules = validation[key];
        var _rule = null;
        if (rules.find(rule => {
            if (!stripeState) {
                return true;
            }
            var _validationStateValue = getValue(stripeState, key);
            if (stripeState.hasOwnProperty(key) && !rule.func(_validationStateValue, stripeState)) {
                _rule = rule;
                return true;
            }
            return false;
        })) {
            validationState = Object.assign(validationState, { [key]: _rule ? _rule.message : Titles.Required })
            result.result = false;
        }
        else {
            validationState = Object.assign(validationState, { [key]: false })
        }
    }
    result.validationState = validationState;
    return result;
}